/**
 * \file mkfreqmap.cpp
 *
 * \author deong
 * \date 03/27/2008
 */

#include <iostream>
#include <fstream>
#include <cctype>
#include <cstdlib>
#include <cstring>
#include <iomanip>

using namespace std;

#define CHARS 128

// keep track of how many of each character follow each character
int freqmap[CHARS][CHARS];
	

void process_file(char*);
void write_results();
bool interesting(char);

/**
 * \brief main function
 *
 * read all files specified on the command line and build up the character
 * adjacency frequency map.
 *
 * \author deong
 * \date 03/27/2008
 */
int main(int argc,char** argv)
{
	if(argc==1)
	{
		cerr << "usage: mkfreqmap [filename]+" << endl;
		exit(EXIT_FAILURE);
	}

	for(int i=0; i<CHARS; i++)
		memset(freqmap[i],0,CHARS);
	
	for(int i=1; i<argc; i++)
	{
		process_file(argv[i]);
	}

	write_results();
	return 0;
}

/**
 * \brief read a file character by character and update the frequency map
 *
 * \author deong
 * \date 03/27/2008
 */
void process_file(char* filename)
{
	char cur;
	char prev;
	ifstream in(filename);

	// read the first character of the file
	in >> prev;
	while(!interesting(prev))
		in >> prev;
	prev=tolower(prev);
	
	while(!in.eof())
	{
		in >> cur;
		if(isspace(cur))
			continue;
		cur=tolower(cur);

		freqmap[static_cast<int>(prev)][static_cast<int>(cur)]++;
		prev=cur;
	}
}

/**
 * \brief write out the character frequency matrix
 *
 * \author deong
 * \date 03/27/2008
 */
void write_results()
{
	cout << setw(6) << " ";		// leave a blank to align columns
	for(int i=0; i<CHARS; i++)
	{
		if(interesting(static_cast<char>(i)))
		{
			cout << setw(6) << static_cast<char>(i);
		}
	}
	cout << "\n";
	
	for(int i=0; i<CHARS; i++)
	{
		if(interesting(static_cast<char>(i)))
		{
			cout << setw(6) << static_cast<char>(i);
			for(int j=0; j<CHARS; j++)
			{
				if(interesting(static_cast<char>(j)))
				{
					cout << setw(6) << freqmap[i][j];
				}
			}
			cout << "\n";
		}
	}
	cout << endl;
}

/**
 * \brief return true if a character is "interesting", false otherwise
 *
 * interesting characters are alphanumeric, punctuation
 *
 * \author deong
 * \date 03/27/2008
 */
bool interesting(char c)
{
	return (isprint(c) && !isspace(c) && (!isalpha(c) || islower(c)));
}
